# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: operation-init.j2

"""Auto-generated package that contains models used by the AccelByte Gaming Services Platform Service."""

__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .get_xbl_user_achievements import GetXblUserAchievements
from .unlock_steam_user_achievement import UnlockSteamUserAchievement
from .update_xbl_user_achievement import UpdateXblUserAchievement



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model
from accelbyte_py_sdk.core import ApiError
from accelbyte_py_sdk.core import StrEnum

from ..models.credit_summary import CreditSummary
from ..models.entitlement_summary import EntitlementSummary
from ..models.fulfillment_item import FulfillmentItem
from ..models.fulfillment_state_info import FulfillmentStateInfo
from ..models.subscription_summary import SubscriptionSummary


class StateEnum(StrEnum):
    FULFILLED = "FULFILLED"
    FULFILL_FAILED = "FULFILL_FAILED"
    REVOKED = "REVOKED"
    REVOKE_FAILED = "REVOKE_FAILED"


class FulfillmentV2Result(Model):
    """Fulfillment V2 result (FulfillmentV2Result)

    Properties:
        items: (items) REQUIRED List[FulfillmentItem]

        namespace: (namespace) REQUIRED str

        state: (state) REQUIRED Union[str, StateEnum]

        state_info: (stateInfo) REQUIRED FulfillmentStateInfo

        transaction_id: (transactionId) REQUIRED str

        user_id: (userId) REQUIRED str

        credit_summaries: (creditSummaries) OPTIONAL List[CreditSummary]

        entitlement_summaries: (entitlementSummaries) OPTIONAL List[EntitlementSummary]

        id_: (id) OPTIONAL str

        state_reason: (stateReason) OPTIONAL str

        subscription_summaries: (subscriptionSummaries) OPTIONAL List[SubscriptionSummary]
    """

    # region fields

    items: List[FulfillmentItem]  # REQUIRED
    namespace: str  # REQUIRED
    state: Union[str, StateEnum]  # REQUIRED
    state_info: FulfillmentStateInfo  # REQUIRED
    transaction_id: str  # REQUIRED
    user_id: str  # REQUIRED
    credit_summaries: List[CreditSummary]  # OPTIONAL
    entitlement_summaries: List[EntitlementSummary]  # OPTIONAL
    id_: str  # OPTIONAL
    state_reason: str  # OPTIONAL
    subscription_summaries: List[SubscriptionSummary]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_items(self, value: List[FulfillmentItem]) -> FulfillmentV2Result:
        self.items = value
        return self

    def with_namespace(self, value: str) -> FulfillmentV2Result:
        self.namespace = value
        return self

    def with_state(self, value: Union[str, StateEnum]) -> FulfillmentV2Result:
        self.state = value
        return self

    def with_state_info(self, value: FulfillmentStateInfo) -> FulfillmentV2Result:
        self.state_info = value
        return self

    def with_transaction_id(self, value: str) -> FulfillmentV2Result:
        self.transaction_id = value
        return self

    def with_user_id(self, value: str) -> FulfillmentV2Result:
        self.user_id = value
        return self

    def with_credit_summaries(self, value: List[CreditSummary]) -> FulfillmentV2Result:
        self.credit_summaries = value
        return self

    def with_entitlement_summaries(
        self, value: List[EntitlementSummary]
    ) -> FulfillmentV2Result:
        self.entitlement_summaries = value
        return self

    def with_id(self, value: str) -> FulfillmentV2Result:
        self.id_ = value
        return self

    def with_state_reason(self, value: str) -> FulfillmentV2Result:
        self.state_reason = value
        return self

    def with_subscription_summaries(
        self, value: List[SubscriptionSummary]
    ) -> FulfillmentV2Result:
        self.subscription_summaries = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "items"):
            result["items"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.items
            ]
        elif include_empty:
            result["items"] = []
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "state"):
            result["state"] = str(self.state)
        elif include_empty:
            result["state"] = Union[str, StateEnum]()
        if hasattr(self, "state_info"):
            result["stateInfo"] = self.state_info.to_dict(include_empty=include_empty)
        elif include_empty:
            result["stateInfo"] = FulfillmentStateInfo()
        if hasattr(self, "transaction_id"):
            result["transactionId"] = str(self.transaction_id)
        elif include_empty:
            result["transactionId"] = ""
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "credit_summaries"):
            result["creditSummaries"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.credit_summaries
            ]
        elif include_empty:
            result["creditSummaries"] = []
        if hasattr(self, "entitlement_summaries"):
            result["entitlementSummaries"] = [
                i0.to_dict(include_empty=include_empty)
                for i0 in self.entitlement_summaries
            ]
        elif include_empty:
            result["entitlementSummaries"] = []
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "state_reason"):
            result["stateReason"] = str(self.state_reason)
        elif include_empty:
            result["stateReason"] = ""
        if hasattr(self, "subscription_summaries"):
            result["subscriptionSummaries"] = [
                i0.to_dict(include_empty=include_empty)
                for i0 in self.subscription_summaries
            ]
        elif include_empty:
            result["subscriptionSummaries"] = []
        return result

    def translate_to_api_error(self) -> ApiError:
        return ApiError(
            code="",
            message="",
        )

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        items: List[FulfillmentItem],
        namespace: str,
        state: Union[str, StateEnum],
        state_info: FulfillmentStateInfo,
        transaction_id: str,
        user_id: str,
        credit_summaries: Optional[List[CreditSummary]] = None,
        entitlement_summaries: Optional[List[EntitlementSummary]] = None,
        id_: Optional[str] = None,
        state_reason: Optional[str] = None,
        subscription_summaries: Optional[List[SubscriptionSummary]] = None,
        **kwargs,
    ) -> FulfillmentV2Result:
        instance = cls()
        instance.items = items
        instance.namespace = namespace
        instance.state = state
        instance.state_info = state_info
        instance.transaction_id = transaction_id
        instance.user_id = user_id
        if credit_summaries is not None:
            instance.credit_summaries = credit_summaries
        if entitlement_summaries is not None:
            instance.entitlement_summaries = entitlement_summaries
        if id_ is not None:
            instance.id_ = id_
        if state_reason is not None:
            instance.state_reason = state_reason
        if subscription_summaries is not None:
            instance.subscription_summaries = subscription_summaries
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> FulfillmentV2Result:
        instance = cls()
        if not dict_:
            return instance
        if "items" in dict_ and dict_["items"] is not None:
            instance.items = [
                FulfillmentItem.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["items"]
            ]
        elif include_empty:
            instance.items = []
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "state" in dict_ and dict_["state"] is not None:
            instance.state = str(dict_["state"])
        elif include_empty:
            instance.state = Union[str, StateEnum]()
        if "stateInfo" in dict_ and dict_["stateInfo"] is not None:
            instance.state_info = FulfillmentStateInfo.create_from_dict(
                dict_["stateInfo"], include_empty=include_empty
            )
        elif include_empty:
            instance.state_info = FulfillmentStateInfo()
        if "transactionId" in dict_ and dict_["transactionId"] is not None:
            instance.transaction_id = str(dict_["transactionId"])
        elif include_empty:
            instance.transaction_id = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "creditSummaries" in dict_ and dict_["creditSummaries"] is not None:
            instance.credit_summaries = [
                CreditSummary.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["creditSummaries"]
            ]
        elif include_empty:
            instance.credit_summaries = []
        if (
            "entitlementSummaries" in dict_
            and dict_["entitlementSummaries"] is not None
        ):
            instance.entitlement_summaries = [
                EntitlementSummary.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["entitlementSummaries"]
            ]
        elif include_empty:
            instance.entitlement_summaries = []
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "stateReason" in dict_ and dict_["stateReason"] is not None:
            instance.state_reason = str(dict_["stateReason"])
        elif include_empty:
            instance.state_reason = ""
        if (
            "subscriptionSummaries" in dict_
            and dict_["subscriptionSummaries"] is not None
        ):
            instance.subscription_summaries = [
                SubscriptionSummary.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["subscriptionSummaries"]
            ]
        elif include_empty:
            instance.subscription_summaries = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, FulfillmentV2Result]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[FulfillmentV2Result]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        FulfillmentV2Result, List[FulfillmentV2Result], Dict[Any, FulfillmentV2Result]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "items": "items",
            "namespace": "namespace",
            "state": "state",
            "stateInfo": "state_info",
            "transactionId": "transaction_id",
            "userId": "user_id",
            "creditSummaries": "credit_summaries",
            "entitlementSummaries": "entitlement_summaries",
            "id": "id_",
            "stateReason": "state_reason",
            "subscriptionSummaries": "subscription_summaries",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "items": True,
            "namespace": True,
            "state": True,
            "stateInfo": True,
            "transactionId": True,
            "userId": True,
            "creditSummaries": False,
            "entitlementSummaries": False,
            "id": False,
            "stateReason": False,
            "subscriptionSummaries": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "state": ["FULFILLED", "FULFILL_FAILED", "REVOKED", "REVOKE_FAILED"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model

from ..models.platform_reward import PlatformReward
from ..models.reward_migration_result import RewardMigrationResult


class EntitlementOriginSyncResult(Model):
    """entitlement origin sync result (EntitlementOriginSyncResult)

    Properties:
        reason: (reason) OPTIONAL str

        reward: (reward) OPTIONAL PlatformReward

        summary: (summary) OPTIONAL List[RewardMigrationResult]
    """

    # region fields

    reason: str  # OPTIONAL
    reward: PlatformReward  # OPTIONAL
    summary: List[RewardMigrationResult]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_reason(self, value: str) -> EntitlementOriginSyncResult:
        self.reason = value
        return self

    def with_reward(self, value: PlatformReward) -> EntitlementOriginSyncResult:
        self.reward = value
        return self

    def with_summary(
        self, value: List[RewardMigrationResult]
    ) -> EntitlementOriginSyncResult:
        self.summary = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "reason"):
            result["reason"] = str(self.reason)
        elif include_empty:
            result["reason"] = ""
        if hasattr(self, "reward"):
            result["reward"] = self.reward.to_dict(include_empty=include_empty)
        elif include_empty:
            result["reward"] = PlatformReward()
        if hasattr(self, "summary"):
            result["summary"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.summary
            ]
        elif include_empty:
            result["summary"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        reason: Optional[str] = None,
        reward: Optional[PlatformReward] = None,
        summary: Optional[List[RewardMigrationResult]] = None,
        **kwargs,
    ) -> EntitlementOriginSyncResult:
        instance = cls()
        if reason is not None:
            instance.reason = reason
        if reward is not None:
            instance.reward = reward
        if summary is not None:
            instance.summary = summary
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> EntitlementOriginSyncResult:
        instance = cls()
        if not dict_:
            return instance
        if "reason" in dict_ and dict_["reason"] is not None:
            instance.reason = str(dict_["reason"])
        elif include_empty:
            instance.reason = ""
        if "reward" in dict_ and dict_["reward"] is not None:
            instance.reward = PlatformReward.create_from_dict(
                dict_["reward"], include_empty=include_empty
            )
        elif include_empty:
            instance.reward = PlatformReward()
        if "summary" in dict_ and dict_["summary"] is not None:
            instance.summary = [
                RewardMigrationResult.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["summary"]
            ]
        elif include_empty:
            instance.summary = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, EntitlementOriginSyncResult]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[EntitlementOriginSyncResult]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        EntitlementOriginSyncResult,
        List[EntitlementOriginSyncResult],
        Dict[Any, EntitlementOriginSyncResult],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "reason": "reason",
            "reward": "reward",
            "summary": "summary",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "reason": False,
            "reward": False,
            "summary": False,
        }

    # endregion static methods



# Copyright (c) 2024 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: errors.j2

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Gaming Services Platform Service

from accelbyte_py_sdk.core import ApiError

ERROR_20002 = ApiError(code="20002", message="validation error")
ERROR_20006 = ApiError(code="20006", message="optimistic lock")
ERROR_20007 = ApiError(code="20007", message="too many requests")
ERROR_20008 = ApiError(
    code="20008", message="user [{userId}] does not exist in namespace [{namespace}]"
)
ERROR_20016 = ApiError(code="20016", message="action is banned")
ERROR_20018 = ApiError(code="20018", message="ecommerce item type not supported")
ERROR_20024 = ApiError(
    code="20024", message="insufficient inventory capacity (max. slots)"
)
ERROR_20027 = ApiError(code="20027", message="Invalid time range")
ERROR_30021 = ApiError(code="30021", message="Default language [{language}] required")
ERROR_30022 = ApiError(code="30022", message="Default region [{region}] is required")
ERROR_30023 = ApiError(
    code="30023", message="Catalog plugin grpc server address required"
)
ERROR_30024 = ApiError(code="30024", message="Unable to parse CSV cell [{content}]")
ERROR_30025 = ApiError(
    code="30025",
    message="[{header}] is required by CSV import/export for catalogType [{catalogType}]",
)
ERROR_30041 = ApiError(
    code="30041",
    message="Changelog [{changelogId}] does not exist in namespace [{namespace}]",
)
ERROR_30071 = ApiError(
    code="30071",
    message="Can't unselect item [{itemId}] when the item which is bound to is already selected in namespace [{namespace}]",
)
ERROR_30072 = ApiError(
    code="30072",
    message="Can't unselect category [{categoryPath}] when item with this category is already selected in namespace [{namespace}]",
)
ERROR_30073 = ApiError(code="30073", message="Can't unselect store change")
ERROR_30074 = ApiError(
    code="30074",
    message="Can't unselect subscription's content [{itemId}] when subscription is already selected in namespace [{namespace}]",
)
ERROR_30076 = ApiError(
    code="30076",
    message="CSV header [{headerName}] is not supported for CatalogType [{catalogType}]",
)
ERROR_30121 = ApiError(code="30121", message="Store data is invalid")
ERROR_30122 = ApiError(code="30122", message="Store's meta mismatch")
ERROR_30141 = ApiError(
    code="30141", message="Store [{storeId}] does not exist in namespace [{namespace}]"
)
ERROR_30142 = ApiError(
    code="30142", message="Published store does not exist in namespace [{namespace}]"
)
ERROR_30143 = ApiError(
    code="30143",
    message="Published store [{storeId}] backup does not exist in namespace [{namespace}]",
)
ERROR_30171 = ApiError(
    code="30171",
    message="Store [{store}] can't change default language to [{language}]",
)
ERROR_30172 = ApiError(
    code="30172", message="Store [{store}] can't change default region to [{region}]"
)
ERROR_30173 = ApiError(code="30173", message="Published store can't modify content")
ERROR_30174 = ApiError(
    code="30174", message="Draft store already exists in namespace [{namespace}]"
)
ERROR_30175 = ApiError(
    code="30175",
    message="Duplicated currencyCode [{currencyCode}] in Region [{region}]",
)
ERROR_30241 = ApiError(
    code="30241",
    message="Category [{categoryPath}] does not exist in namespace [{namespace}]",
)
ERROR_30271 = ApiError(
    code="30271",
    message="Category [{categoryPath}] already exists in namespace [{namespace}]",
)
ERROR_30272 = ApiError(
    code="30272",
    message="Category [{categoryPath}] is not empty in namespace [{namespace}]",
)
ERROR_30301 = ApiError(
    code="30301",
    message="Unsupported Item Type [{itemType}] for box item [{itemId}] with expiration",
)
ERROR_30321 = ApiError(code="30321", message="Invalid item discount amount")
ERROR_30322 = ApiError(code="30322", message="Bundle item [{itemId}] can't be bundled")
ERROR_30323 = ApiError(code="30323", message="Target namespace is required")
ERROR_30324 = ApiError(code="30324", message="Invalid namespace [{namespace}]")
ERROR_30325 = ApiError(code="30325", message="Code item [{itemId}] can't be bundled")
ERROR_30326 = ApiError(
    code="30326", message="Subscription item [{itemId}] can't be bundled"
)
ERROR_30327 = ApiError(code="30327", message="Invalid item trial price")
ERROR_30329 = ApiError(code="30329", message="Invalid bundled item [{itemId}] quantity")
ERROR_30330 = ApiError(
    code="30330", message="Invalid item region price currency namespace [{namespace}]"
)
ERROR_30331 = ApiError(code="30331", message="Invalid purchase condition")
ERROR_30332 = ApiError(
    code="30332", message="Invalid option box item [{itemId}] quantity"
)
ERROR_30333 = ApiError(
    code="30333",
    message="Item [{itemId}] item type [{itemType}] can't be bundled into option box",
)
ERROR_30334 = ApiError(
    code="30334", message="Option box item [{itemId}] can't be bundled"
)
ERROR_30335 = ApiError(
    code="30335",
    message="Item [{itemId}] can't be deleted in non-forced mode if item has been published",
)
ERROR_30336 = ApiError(code="30336", message="Item type [{itemType}] does not support")
ERROR_30337 = ApiError(
    code="30337", message="Invalid loot box item [{itemId}] quantity"
)
ERROR_30338 = ApiError(
    code="30338",
    message="Item [{itemId}] item type [{itemType}] can't be bundled into loot box",
)
ERROR_30339 = ApiError(
    code="30339", message="Loot box item [{itemId}] can't be bundled"
)
ERROR_30341 = ApiError(
    code="30341", message="Item [{itemId}] does not exist in namespace [{namespace}]"
)
ERROR_30342 = ApiError(code="30342", message="Item of appId [{appId}] does not exist")
ERROR_30343 = ApiError(code="30343", message="Item of sku [{sku}] does not exist")
ERROR_30371 = ApiError(code="30371", message="Item maxCount not allow reduce")
ERROR_30372 = ApiError(code="30372", message="ItemType is not updatable")
ERROR_30373 = ApiError(
    code="30373",
    message="ItemType [{itemType}] is not allowed in namespace [{namespace}]",
)
ERROR_30374 = ApiError(
    code="30374", message="Item sku [{sku}] already exists in namespace [{namespace}]"
)
ERROR_30375 = ApiError(
    code="30375",
    message="Item id [{itemId}] of sku [{sku}] is duplicate with un-published deleted item in namespace [{namespace}]",
)
ERROR_30376 = ApiError(
    code="30376", message="Publisher namespace don’t allow sellback item"
)
ERROR_30377 = ApiError(
    code="30377", message="This item type [{itemType}] don’t allow sellback"
)
ERROR_30378 = ApiError(
    code="30378", message="Sale price don’t allow real currency [{currencyCode}]"
)
ERROR_30379 = ApiError(code="30379", message="Item sku is not updatable")
ERROR_30380 = ApiError(
    code="30380",
    message="Box item [{itemId}] duration and end date can’t be set at the same time",
)
ERROR_30381 = ApiError(
    code="30381",
    message="Currency [{currency}] is not set for bundle Item [{itemId}] in region [{region}]",
)
ERROR_30382 = ApiError(code="30382", message="Duplicated Item sku [{sku}]")
ERROR_30383 = ApiError(
    code="30383",
    message="Item app id [{appId}] already exists in namespace [{namespace}] item [{itemId}]",
)
ERROR_30386 = ApiError(
    code="30386",
    message="The item [{itemId}] is currently associated and cannot be deleted in namespace [{namespace}], Feature {featureName}, Module {moduleName}, and Reference ID {referenceId} are using this item ID",
)
ERROR_30387 = ApiError(
    code="30387",
    message="The item [{itemId}] is currently associated and cannot be disabled in namespace [{namespace}], Feature {featureName}, Module {moduleName}, and Reference ID {referenceId} are using this item ID",
)
ERROR_30541 = ApiError(code="30541", message="Item type config [{id}] doesn't exist")
ERROR_30641 = ApiError(
    code="30641", message="View [{viewId}] does not exist in namespace [{namespace}]"
)
ERROR_30741 = ApiError(
    code="30741",
    message="Section [{sectionId}] does not exist in namespace [{namespace}]",
)
ERROR_30771 = ApiError(
    code="30771",
    message="Item [{itemId}] not found in User Section [{sectionId}], UserId [{userId}], Namespace [{namespace}]",
)
ERROR_30772 = ApiError(
    code="30772", message="Section [{sectionId}] is not available or expired"
)
ERROR_31121 = ApiError(
    code="31121", message="OptionBox entitlement [{entitlementId}] use count is not 1"
)
ERROR_31122 = ApiError(
    code="31122",
    message="OptionBox entitlement [{entitlementId}] options size is not 1",
)
ERROR_31123 = ApiError(
    code="31123", message="Unable to acquire box item, box item [{itemId}] expired"
)
ERROR_31141 = ApiError(
    code="31141",
    message="Entitlement [{entitlementId}] does not exist in namespace [{namespace}]",
)
ERROR_31142 = ApiError(
    code="31142",
    message="Entitlement with appId [{appId}] does not exist in namespace [{namespace}]",
)
ERROR_31143 = ApiError(
    code="31143",
    message="Entitlement with sku [{sku}] does not exist in namespace [{namespace}]",
)
ERROR_31144 = ApiError(
    code="31144",
    message="Entitlement with itemId [{itemId}] does not exist in namespace [{namespace}]",
)
ERROR_31145 = ApiError(
    code="31145",
    message="Option [{option}] doesn't exist in OptionBox entitlement [{entitlementId}]",
)
ERROR_31147 = ApiError(
    code="31147", message="Origin [Steam] and System need exist in allowPlatformOrigin"
)
ERROR_31171 = ApiError(
    code="31171", message="Entitlement [{entitlementId}] already revoked"
)
ERROR_31172 = ApiError(code="31172", message="Entitlement [{entitlementId}] not active")
ERROR_31173 = ApiError(
    code="31173", message="Entitlement [{entitlementId}] is not consumable"
)
ERROR_31174 = ApiError(
    code="31174", message="Entitlement [{entitlementId}] already consumed"
)
ERROR_31176 = ApiError(
    code="31176", message="Entitlement [{entitlementId}] use count is insufficient"
)
ERROR_31177 = ApiError(code="31177", message="Permanent item already owned")
ERROR_31178 = ApiError(
    code="31178", message="Entitlement [{entitlementId}] out of time range"
)
ERROR_31179 = ApiError(code="31179", message="Duplicate entitlement exists")
ERROR_31180 = ApiError(code="31180", message="Duplicate request id: [{requestId}]")
ERROR_31181 = ApiError(
    code="31181", message="Entitlement [{entitlementId}] is not sellable"
)
ERROR_31182 = ApiError(
    code="31182", message="Entitlement [{entitlementId}] already sold"
)
ERROR_31183 = ApiError(
    code="31183",
    message="Entitlement [{entitlementId}] origin [{origin}] not allowed be operated at [{platform}]",
)
ERROR_31184 = ApiError(
    code="31184",
    message="Source entitlement [{sourceEntitlementId}] and target entitlement [{targetEntitlementId}] should have same collectionId, timeRange, origin and itemId",
)
ERROR_31185 = ApiError(
    code="31185",
    message="Transferred source entitlement [{sourceEntitlementId}] and target entitlement [{targetEntitlementId}] can not be set to same",
)
ERROR_32121 = ApiError(code="32121", message="Order price mismatch")
ERROR_32122 = ApiError(code="32122", message="Item type [{itemType}] does not support")
ERROR_32123 = ApiError(code="32123", message="Item is not purchasable")
ERROR_32124 = ApiError(code="32124", message="Invalid currency namespace")
ERROR_32125 = ApiError(
    code="32125", message="The user does not meet the purchase conditions"
)
ERROR_32126 = ApiError(
    code="32126", message="Section ID is required for placing this order"
)
ERROR_32127 = ApiError(
    code="32127", message="Discount code [{code}] can't be used on this item: {tips}"
)
ERROR_32128 = ApiError(
    code="32128",
    message="Discount code [{code}] can not be used with other code together",
)
ERROR_32129 = ApiError(code="32129", message="Can't use discount code on free order")
ERROR_32130 = ApiError(
    code="32130", message="The total discount amount cannot exceed the order price"
)
ERROR_32141 = ApiError(code="32141", message="Order [{orderNo}] does not exist")
ERROR_32171 = ApiError(code="32171", message="Order [{orderNo}] is not refundable")
ERROR_32172 = ApiError(
    code="32172", message="Invalid order status [{status}] for order [{orderNo}]"
)
ERROR_32173 = ApiError(
    code="32173", message="Receipt of order [{orderNo}] is not downloadable"
)
ERROR_32175 = ApiError(
    code="32175", message="Exceed item [{itemId}] max count [{maxCount}] per user"
)
ERROR_32176 = ApiError(
    code="32176", message="Exceed item [{itemId}] max count [{maxCount}]"
)
ERROR_32177 = ApiError(code="32177", message="Order [{orderNo}] is not cancelable")
ERROR_32178 = ApiError(
    code="32178",
    message="User [{userId}] already owned all durable items in flexible bundle [{bundleId}], namespace: [{namespace}]",
)
ERROR_33045 = ApiError(
    code="33045",
    message="errors.net.accelbyte.platform.payment.payment_merchant_config_not_found",
)
ERROR_33121 = ApiError(
    code="33121",
    message="Recurring payment failed with code: [{errorCode}] and message: [{errorMessage}] by provider: [{provider}]",
)
ERROR_33122 = ApiError(
    code="33122", message="Subscription not match when create payment order"
)
ERROR_33123 = ApiError(code="33123", message="Invalid zipcode")
ERROR_33141 = ApiError(
    code="33141", message="Payment Order [{paymentOrderNo}] does not exist"
)
ERROR_33145 = ApiError(code="33145", message="Recurring token not found")
ERROR_33171 = ApiError(
    code="33171",
    message="Invalid payment order status [{status}] for payment order [{paymentOrderNo}]",
)
ERROR_33172 = ApiError(
    code="33172", message="Payment order [{paymentOrderNo}] is not refundable"
)
ERROR_33173 = ApiError(
    code="33173",
    message="ExtOrderNo [{extOrderNo}] already exists in namespace [{namespace}]",
)
ERROR_33221 = ApiError(code="33221", message="TaxJar api token required")
ERROR_33241 = ApiError(
    code="33241", message="Payment provider config [{id}] does not exist"
)
ERROR_33242 = ApiError(
    code="33242", message="Payment merchant config [{id}] does not exist"
)
ERROR_33243 = ApiError(
    code="33243", message="Payment callback config for [{namespace}] does not exist"
)
ERROR_33271 = ApiError(
    code="33271",
    message="Payment provider config for namespace [{namespace}] and region [{region}] already exists",
)
ERROR_33321 = ApiError(
    code="33321",
    message="Payment provider [{paymentProvider}] not support currency [{currency}]",
)
ERROR_33322 = ApiError(
    code="33322", message="Payment provider [{paymentProvider}] not supported"
)
ERROR_33332 = ApiError(
    code="33332", message="Amount too small, please contact administrator"
)
ERROR_33333 = ApiError(
    code="33333",
    message="Neon Pay checkout payment order [{paymentOrderNo}] failed with message [{errMsg}]",
)
ERROR_34021 = ApiError(
    code="34021", message="Reward data for namespace [{namespace}] is invalid"
)
ERROR_34023 = ApiError(
    code="34023",
    message="Reward Item [{itemId}] with item type [{itemType}] is not supported for duration or endDate",
)
ERROR_34027 = ApiError(
    code="34027",
    message="Reward Item [{sku}] with item type [{itemType}] is not supported for duration or endDate",
)
ERROR_34041 = ApiError(
    code="34041",
    message="Reward [{rewardId}] does not exist in namespace [{namespace}]",
)
ERROR_34042 = ApiError(
    code="34042",
    message="Reward item [{itemId}] does not exist in namespace [{namespace}]",
)
ERROR_34043 = ApiError(
    code="34043",
    message="Reward with code [{rewardCode}] does not exist in namespace [{namespace}]",
)
ERROR_34044 = ApiError(
    code="34044",
    message="Reward item [{sku}] does not exist in namespace [{namespace}]",
)
ERROR_34071 = ApiError(
    code="34071",
    message="Reward with code [{rewardCode}] already exists in namespace [{namespace}]",
)
ERROR_34072 = ApiError(
    code="34072",
    message="Duplicate reward condition [{rewardConditionName}] found in reward [{rewardCode}]",
)
ERROR_34074 = ApiError(
    code="34074",
    message="Reward Item [{itemId}] duration and end date can’t be set at the same time",
)
ERROR_34076 = ApiError(
    code="34076",
    message="Reward Item [{sku}] duration and end date can’t be set at the same time",
)
ERROR_35123 = ApiError(code="35123", message="Wallet [{walletId}] is inactive")
ERROR_35124 = ApiError(
    code="35124", message="Wallet [{currencyCode}] has insufficient balance"
)
ERROR_35141 = ApiError(code="35141", message="Wallet [{walletId}] does not exist")
ERROR_36141 = ApiError(
    code="36141",
    message="Currency [{currencyCode}] does not exist in namespace [{namespace}]",
)
ERROR_36171 = ApiError(
    code="36171",
    message="Currency [{currencyCode}] already exists in namespace [{namespace}]",
)
ERROR_36172 = ApiError(
    code="36172",
    message="Real Currency [{currencyCode}] not allowed in game namespace [{namespace}]",
)
ERROR_37041 = ApiError(
    code="37041",
    message="Ticket booth [{boothName}] does not exist in namespace [{namespace}]",
)
ERROR_37071 = ApiError(
    code="37071",
    message="Insufficient ticket in booth [{boothName}] in namespace [{namespace}]",
)
ERROR_37121 = ApiError(
    code="37121",
    message="Invalid currency namespace [{namespace}] in discount config: {tips}",
)
ERROR_37141 = ApiError(
    code="37141",
    message="Campaign [{campaignId}] does not exist in namespace [{namespace}]",
)
ERROR_37142 = ApiError(
    code="37142", message="Code [{code}] does not exist in namespace [{namespace}]"
)
ERROR_37143 = ApiError(
    code="37143",
    message="Batch name [{batchName}] does not exist for campaign [{campaignId}] in namespace [{namespace}].",
)
ERROR_37144 = ApiError(
    code="37144",
    message="Campaign batch name does not exist for batch number [{batchNo}] campaign [{campaignId}] in namespace [{namespace}].",
)
ERROR_37171 = ApiError(
    code="37171", message="Campaign [{name}] already exists in namespace [{namespace}]"
)
ERROR_37172 = ApiError(
    code="37172",
    message="Campaign [{campaignId}] is inactive in namespace [{namespace}]",
)
ERROR_37173 = ApiError(
    code="37173", message="Code [{code}] is inactive in namespace [{namespace}]"
)
ERROR_37174 = ApiError(
    code="37174", message="Exceeded max redeem count per code [{maxCount}]"
)
ERROR_37175 = ApiError(
    code="37175", message="Exceeded max redeem count per code per user [{maxCount}]"
)
ERROR_37176 = ApiError(
    code="37176", message="Code [{code}] has been redeemed in namespace [{namespace}]"
)
ERROR_37177 = ApiError(code="37177", message="Code redemption not started")
ERROR_37178 = ApiError(code="37178", message="Code redemption already ended")
ERROR_37179 = ApiError(
    code="37179", message="Exceeded max redeem count per campaign per user [{maxCount}]"
)
ERROR_37180 = ApiError(
    code="37180", message="Code [{code}] already exists in namespace [{namespace}]"
)
ERROR_37221 = ApiError(code="37221", message="Invalid key file")
ERROR_37241 = ApiError(
    code="37241",
    message="Key group [{keyGroupId}] does not exist in namespace [{namespace}]",
)
ERROR_37271 = ApiError(
    code="37271", message="Key group [{name}] already exists in namespace [{namespace}]"
)
ERROR_38121 = ApiError(code="38121", message="Duplicate permanent item exists")
ERROR_38122 = ApiError(code="38122", message="Subscription endDate required")
ERROR_38128 = ApiError(
    code="38128",
    message="Cannot retry fulfillment with different payload. Please check the items list.",
)
ERROR_38129 = ApiError(
    code="38129",
    message="Cannot combine same item [{itemId}] with different [{fieldName}] value",
)
ERROR_38130 = ApiError(
    code="38130",
    message="Cannot fulfill item with type [{itemType}] in item [{itemIdentity}]",
)
ERROR_38141 = ApiError(code="38141", message="Fulfillment script does not exist")
ERROR_38145 = ApiError(
    code="38145",
    message="Fulfillment with transactionId [{transactionId}] does not exist",
)
ERROR_38171 = ApiError(code="38171", message="Fulfillment script already exists")
ERROR_39121 = ApiError(
    code="39121",
    message="Apple iap receipt verify failed with status code [{statusCode}]",
)
ERROR_39122 = ApiError(
    code="39122",
    message="Google iap receipt is invalid with status code [{statusCode}] and error message [{message}]",
)
ERROR_39123 = ApiError(code="39123", message="IAP request is not in valid application")
ERROR_39124 = ApiError(
    code="39124",
    message="IAP request platform [{platformId}] user id is not linked with current user",
)
ERROR_39125 = ApiError(
    code="39125", message="Invalid platform [{platformId}] user token"
)
ERROR_39126 = ApiError(
    code="39126", message="User id [{}] in namespace [{}] doesn't link platform [{}]"
)
ERROR_39127 = ApiError(code="39127", message="Invalid service label [{serviceLabel}]")
ERROR_39128 = ApiError(code="39128", message="Steam publisher key is invalid")
ERROR_39129 = ApiError(code="39129", message="Steam app id is invalid")
ERROR_39130 = ApiError(code="39130", message="Invalid playstation config: [{message}]")
ERROR_39131 = ApiError(
    code="39131",
    message="Invalid Apple IAP config under namespace [{namespace}]: [{message}]",
)
ERROR_39132 = ApiError(
    code="39132",
    message="Bad request for playstation under namespace [{namespace}], reason: [{reason}].",
)
ERROR_39133 = ApiError(code="39133", message="Bad request for Oculus: [{reason}]")
ERROR_39134 = ApiError(
    code="39134",
    message="Invalid Oculus IAP config under namespace [{namespace}]: [{message}]",
)
ERROR_39135 = ApiError(
    code="39135",
    message="Invalid Google IAP config under namespace [{namespace}]: [{message}]",
)
ERROR_39136 = ApiError(
    code="39136",
    message="Request Apple API failed with status code [{statusCode}] and error message [{message}]",
)
ERROR_39137 = ApiError(
    code="39137",
    message="Verify Apple transaction failed with status [{status}] and error message [{message}]",
)
ERROR_39138 = ApiError(
    code="39138",
    message="Apple IAP version mismatch detected: The current configuration is set to  [{configVersion}], but the API version is [{apiVersion}]. Please ensure that both the configuration and API versions are aligned",
)
ERROR_39141 = ApiError(
    code="39141",
    message="Apple iap receipt of transaction [{transactionId}] for productId [{}] does not exist",
)
ERROR_39142 = ApiError(
    code="39142", message="Apple IAP config not found in namespace [{namespace}]"
)
ERROR_39143 = ApiError(
    code="39143", message="PlayStation IAP config not found in namespace [{namespace}]"
)
ERROR_39144 = ApiError(
    code="39144", message="Steam IAP config not found in namespace [{namespace}]."
)
ERROR_39145 = ApiError(
    code="39145", message="XBox IAP config not found in namespace [{namespace}]."
)
ERROR_39146 = ApiError(
    code="39146", message="Oculus IAP config not found in namespace [{namespace}]."
)
ERROR_39147 = ApiError(
    code="39147", message="Epic IAP config not found in namespace [{namespace}]."
)
ERROR_39148 = ApiError(
    code="39148", message="Google IAP config not found in namespace [{namespace}]."
)
ERROR_39149 = ApiError(
    code="39149",
    message="Third Party Subscription Transaction [{id}] not found for user [{userId}] in the namespace [{namespace}].",
)
ERROR_39150 = ApiError(
    code="39150",
    message="Third Party User Subscription [{id}] not found for user [{userId}] in the namespace [{namespace}]..",
)
ERROR_39151 = ApiError(
    code="39151",
    message="IAP order no [{iapOrderNo}] not found in namespace [{namespace}].",
)
ERROR_39152 = ApiError(
    code="39152",
    message="Platform [{platform}] - Third-party subscription group [{sku}] not found in namespace [{namespace}]. It may have been deleted.",
)
ERROR_39154 = ApiError(
    code="39154",
    message="Meta Quest Subscription Sku [{sku}] not found in namespace [{namespace}] config, please config this Subscription sku in subscription group.",
)
ERROR_39171 = ApiError(
    code="39171",
    message="The bundle id in namespace [{namespace}] expect [{expected}] but was [{actual}]",
)
ERROR_39172 = ApiError(
    code="39172",
    message="The order id in namespace [{namespace}] expect [{expected}] but was [{actual}]",
)
ERROR_39173 = ApiError(
    code="39173",
    message="The purchase status of google play order [{orderId}] in namespace [{namespace}] expect [{expected}] but was [{actual}]",
)
ERROR_39174 = ApiError(
    code="39174",
    message="The google iap purchase time of order [{orderId}] in namespace [{namespace}] expect [{expected}] but was [{actual}]",
)
ERROR_39175 = ApiError(
    code="39175",
    message="Duplicate IAP item mapping, IAPType: [{iapType}] and id: [{iapId}]",
)
ERROR_39183 = ApiError(
    code="39183",
    message="Steam transaction [{orderId}] is still pending or failed, status [{status}], please try it later",
)
ERROR_39184 = ApiError(
    code="39184",
    message="Steam api exception with error code [{errorCode}] and error message [{message}]",
)
ERROR_39185 = ApiError(
    code="39185",
    message="This endpoint only works on sync mode [{workSyncMode}], but current steam iap config sync mode is [{currentSyncMode}] under namespace [{namespace}]",
)
ERROR_39187 = ApiError(
    code="39187",
    message="Duplicated group sku [{sku}] platform [{platform}] in namespace [{namespace}]",
)
ERROR_39188 = ApiError(
    code="39188",
    message="Third-party subscription group [{sku}] on platform [{platform}] in namespace [{namespace}] is already linked to the user's third-party subscription.",
)
ERROR_39221 = ApiError(
    code="39221",
    message="Invalid Xbox Business Partner Certificate or password: [{message}]",
)
ERROR_39244 = ApiError(code="39244", message="Steam config does not exist")
ERROR_39245 = ApiError(code="39245", message="Steam app id does not exist")
ERROR_39321 = ApiError(
    code="39321", message="Invalid IAP item config namespace [{namespace}]: [{message}]"
)
ERROR_39341 = ApiError(
    code="39341", message="IAP item config cannot be found in namespace [{namespace}]"
)
ERROR_39441 = ApiError(
    code="39441",
    message="Platform dlc config cannot be found in namespace [{namespace}]",
)
ERROR_39442 = ApiError(
    code="39442", message="DLC item config cannot be found in namespace [{namespace}]"
)
ERROR_39471 = ApiError(
    code="39471",
    message="Duplicated dlc reward id [{dlcRewardId}] in namespace [{namespace}] ",
)
ERROR_39621 = ApiError(
    code="39621",
    message="Steam api common exception with status code [statusCode] details: [details]",
)
ERROR_40121 = ApiError(code="40121", message="Item type [{itemType}] does not support")
ERROR_40122 = ApiError(
    code="40122", message="Subscription already been subscribed by user"
)
ERROR_40123 = ApiError(
    code="40123", message="Currency [{currencyCode}] does not support"
)
ERROR_40125 = ApiError(
    code="40125",
    message="Subscription [{subscriptionId}] has no real currency billing account",
)
ERROR_40141 = ApiError(
    code="40141", message="Subscription [{subscriptionId}] does not exist"
)
ERROR_40171 = ApiError(
    code="40171", message="Subscription [{subscriptionId}] is not active"
)
ERROR_40172 = ApiError(
    code="40172",
    message="Subscription [{subscriptionId}] is charging, waiting for payment notification",
)
ERROR_40173 = ApiError(
    code="40173",
    message="Subscription [{subscriptionId}] current currency [{currentCurrency}] not match request currency [{requestCurrency}]",
)
ERROR_41171 = ApiError(
    code="41171", message="Request has different payload on previous call"
)
ERROR_41172 = ApiError(
    code="41172", message="Request has different user id on previous call"
)
ERROR_49147 = ApiError(code="49147", message="Published season does not exist")
ERROR_49183 = ApiError(
    code="49183", message="Pass item does not match published season pass"
)
ERROR_49184 = ApiError(
    code="49184", message="Tier item does not match published season tier"
)
ERROR_49185 = ApiError(code="49185", message="Season has not started")
ERROR_49186 = ApiError(code="49186", message="Pass already owned")
ERROR_49187 = ApiError(code="49187", message="Exceed max tier count")
ERROR_394721 = ApiError(
    code="394721",
    message="Invalid platform DLC config namespace [{namespace}]: [{message}]",
)
ERROR_1100001 = ApiError(code="1100001", message="record not found: inventory")



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: operation.j2

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Gaming Services Platform Service

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import ApiError, ApiResponse
from accelbyte_py_sdk.core import Operation
from accelbyte_py_sdk.core import HeaderStr
from accelbyte_py_sdk.core import HttpResponse
from accelbyte_py_sdk.core import deprecated

from ...models import AdminOrderCreate
from ...models import ErrorEntity
from ...models import OrderInfo
from ...models import ValidationErrorEntity


class AdminCreateUserOrder(Operation):
    """Admin Create an order (adminCreateUserOrder)

    Admin Create an order. The result contains the checkout link and payment token. User with permission SANDBOX will create sandbox order that not real paid for xsolla/alipay and not validate price for wxpay.
    Other detail info:

      * It will be forbidden while the user is banned: ORDER_INITIATE or ORDER_AND_PAYMENT
      * sandbox default value is false
      * platform default value is Other
      *  Returns : created order



    ## Restrictions for ext field


    1. Cannot use "." as the key name
    -


        { "data.2": "value" }


    2. Cannot use "$" as the prefix in key names
    -


        { "$data": "value" }

    Properties:
        url: /platform/admin/namespaces/{namespace}/users/{userId}/orders

        method: POST

        tags: ["Order"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED AdminOrderCreate in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        201: Created - OrderInfo (successful operation)

        400: Bad Request - ErrorEntity (20018: ecommerce item type not supported | 32121: Order price mismatch | 32122: Item type [{itemType}] does not support | 32123: Item is not purchasable | 35123: Wallet [{walletId}] is inactive | 35124: Wallet [{currencyCode}] has insufficient balance | 32126: Section ID is required for placing this order | 38121: Duplicate permanent item exists | 32124: Invalid currency namespace | 32127: Discount code [{code}] can't be used on this item: {tips} | 32128: Discount code [{code}] can not be used with other code together | 32129: Can't use discount code on free order | 32130: The total discount amount cannot exceed the order price)

        403: Forbidden - ErrorEntity (20016: action is banned)

        404: Not Found - ErrorEntity (30341: Item [{itemId}] does not exist in namespace [{namespace}] | 30142: Published store does not exist in namespace [{namespace}] | 36141: Currency [{currencyCode}] does not exist in namespace [{namespace}] | 49147: Published season does not exist | 1100001: record not found: inventory | 37142: Code [{code}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (20024: insufficient inventory capacity (max. slots) | 32175: Exceed item [{itemId}] max count [{maxCount}] per user | 32176: Exceed item [{itemId}] max count [{maxCount}] | 32178: User [{userId}] already owned all durable items in flexible bundle [{bundleId}], namespace: [{namespace}] | 31177: Permanent item already owned | 49183: Pass item does not match published season pass | 49184: Tier item does not match published season tier | 49185: Season has not started | 49186: Pass already owned | 49187: Exceed max tier count | 20006: optimistic lock | 37172: Campaign [{campaignId}] is inactive in namespace [{namespace}] | 37173: Code [{code}] is inactive in namespace [{namespace}] | 37174: Exceeded max redeem count per code [{maxCount}] | 37175: Exceeded max redeem count per code per user [{maxCount}] | 37177: Code redemption not started | 37178: Code redemption already ended | 37179: Exceeded max redeem count per campaign per user [{maxCount}] | 30771: Item [{itemId}] not found in User Section [{sectionId}], UserId [{userId}], Namespace [{namespace}] | 30772: Section [{sectionId}] is not available or expired)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """

    # region fields

    _url: str = "/platform/admin/namespaces/{namespace}/users/{userId}/orders"
    _path: str = "/platform/admin/namespaces/{namespace}/users/{userId}/orders"
    _base_path: str = ""
    _method: str = "POST"
    _consumes: List[str] = ["application/json"]
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"]]
    _location_query: str = None

    service_name: Optional[str] = "platform"

    body: AdminOrderCreate  # REQUIRED in [body]
    namespace: str  # REQUIRED in [path]
    user_id: str  # REQUIRED in [path]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def path(self) -> str:
        return self._path

    @property
    def base_path(self) -> str:
        return self._base_path

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "body": self.get_body_params(),
            "path": self.get_path_params(),
        }

    def get_body_params(self) -> Any:
        if not hasattr(self, "body") or self.body is None:
            return None
        return self.body.to_dict()

    def get_path_params(self) -> dict:
        result = {}
        if hasattr(self, "namespace"):
            result["namespace"] = self.namespace
        if hasattr(self, "user_id"):
            result["userId"] = self.user_id
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_body(self, value: AdminOrderCreate) -> AdminCreateUserOrder:
        self.body = value
        return self

    def with_namespace(self, value: str) -> AdminCreateUserOrder:
        self.namespace = value
        return self

    def with_user_id(self, value: str) -> AdminCreateUserOrder:
        self.user_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "body") and self.body:
            result["body"] = self.body.to_dict(include_empty=include_empty)
        elif include_empty:
            result["body"] = AdminOrderCreate()
        if hasattr(self, "namespace") and self.namespace:
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "user_id") and self.user_id:
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        return result

    # endregion to methods

    # region response methods

    class Response(ApiResponse):
        data_201: Optional[OrderInfo] = None
        error_400: Optional[ErrorEntity] = None
        error_403: Optional[ErrorEntity] = None
        error_404: Optional[ErrorEntity] = None
        error_409: Optional[ErrorEntity] = None
        error_422: Optional[ValidationErrorEntity] = None

        def ok(self) -> AdminCreateUserOrder.Response:
            if self.error_400 is not None:
                err = self.error_400.translate_to_api_error()
                exc = err.to_exception()
                if exc is not None:
                    raise exc  # pylint: disable=raising-bad-type
            if self.error_403 is not None:
                err = self.error_403.translate_to_api_error()
                exc = err.to_exception()
                if exc is not None:
                    raise exc  # pylint: disable=raising-bad-type
            if self.error_404 is not None:
                err = self.error_404.translate_to_api_error()
                exc = err.to_exception()
                if exc is not None:
                    raise exc  # pylint: disable=raising-bad-type
            if self.error_409 is not None:
                err = self.error_409.translate_to_api_error()
                exc = err.to_exception()
                if exc is not None:
                    raise exc  # pylint: disable=raising-bad-type
            if self.error_422 is not None:
                err = self.error_422.translate_to_api_error()
                exc = err.to_exception()
                if exc is not None:
                    raise exc  # pylint: disable=raising-bad-type
            return self

        def __iter__(self):
            if self.data_201 is not None:
                yield self.data_201
                yield None
            elif self.error_400 is not None:
                yield None
                yield self.error_400
            elif self.error_403 is not None:
                yield None
                yield self.error_403
            elif self.error_404 is not None:
                yield None
                yield self.error_404
            elif self.error_409 is not None:
                yield None
                yield self.error_409
            elif self.error_422 is not None:
                yield None
                yield self.error_422
            else:
                yield None
                yield self.error

    # noinspection PyMethodMayBeStatic
    def parse_response(self, code: int, content_type: str, content: Any) -> Response:
        """Parse the given response.

        201: Created - OrderInfo (successful operation)

        400: Bad Request - ErrorEntity (20018: ecommerce item type not supported | 32121: Order price mismatch | 32122: Item type [{itemType}] does not support | 32123: Item is not purchasable | 35123: Wallet [{walletId}] is inactive | 35124: Wallet [{currencyCode}] has insufficient balance | 32126: Section ID is required for placing this order | 38121: Duplicate permanent item exists | 32124: Invalid currency namespace | 32127: Discount code [{code}] can't be used on this item: {tips} | 32128: Discount code [{code}] can not be used with other code together | 32129: Can't use discount code on free order | 32130: The total discount amount cannot exceed the order price)

        403: Forbidden - ErrorEntity (20016: action is banned)

        404: Not Found - ErrorEntity (30341: Item [{itemId}] does not exist in namespace [{namespace}] | 30142: Published store does not exist in namespace [{namespace}] | 36141: Currency [{currencyCode}] does not exist in namespace [{namespace}] | 49147: Published season does not exist | 1100001: record not found: inventory | 37142: Code [{code}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (20024: insufficient inventory capacity (max. slots) | 32175: Exceed item [{itemId}] max count [{maxCount}] per user | 32176: Exceed item [{itemId}] max count [{maxCount}] | 32178: User [{userId}] already owned all durable items in flexible bundle [{bundleId}], namespace: [{namespace}] | 31177: Permanent item already owned | 49183: Pass item does not match published season pass | 49184: Tier item does not match published season tier | 49185: Season has not started | 49186: Pass already owned | 49187: Exceed max tier count | 20006: optimistic lock | 37172: Campaign [{campaignId}] is inactive in namespace [{namespace}] | 37173: Code [{code}] is inactive in namespace [{namespace}] | 37174: Exceeded max redeem count per code [{maxCount}] | 37175: Exceeded max redeem count per code per user [{maxCount}] | 37177: Code redemption not started | 37178: Code redemption already ended | 37179: Exceeded max redeem count per campaign per user [{maxCount}] | 30771: Item [{itemId}] not found in User Section [{sectionId}], UserId [{userId}], Namespace [{namespace}] | 30772: Section [{sectionId}] is not available or expired)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        result = AdminCreateUserOrder.Response()

        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )

        if error is not None:
            if not error.is_no_content():
                result.error = ApiError.create_from_http_response(error)
        else:
            code, content_type, content = pre_processed_response

            if code == 201:
                result.data_201 = OrderInfo.create_from_dict(content)
            elif code == 400:
                result.error_400 = ErrorEntity.create_from_dict(content)
                result.error = result.error_400.translate_to_api_error()
            elif code == 403:
                result.error_403 = ErrorEntity.create_from_dict(content)
                result.error = result.error_403.translate_to_api_error()
            elif code == 404:
                result.error_404 = ErrorEntity.create_from_dict(content)
                result.error = result.error_404.translate_to_api_error()
            elif code == 409:
                result.error_409 = ErrorEntity.create_from_dict(content)
                result.error = result.error_409.translate_to_api_error()
            elif code == 422:
                result.error_422 = ValidationErrorEntity.create_from_dict(content)
                result.error = result.error_422.translate_to_api_error()
            else:
                result.error = ApiError.create_from_http_response(
                    HttpResponse.create_undocumented_response(
                        code=code, content_type=content_type, content=content
                    )
                )

        result.status_code = str(code)
        result.content_type = content_type

        if 400 <= code <= 599 or result.error is not None:
            result.is_success = False

        return result

    # noinspection PyMethodMayBeStatic
    @deprecated
    def parse_response_x(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[
        Union[None, OrderInfo],
        Union[None, ErrorEntity, HttpResponse, ValidationErrorEntity],
    ]:
        """Parse the given response.

        201: Created - OrderInfo (successful operation)

        400: Bad Request - ErrorEntity (20018: ecommerce item type not supported | 32121: Order price mismatch | 32122: Item type [{itemType}] does not support | 32123: Item is not purchasable | 35123: Wallet [{walletId}] is inactive | 35124: Wallet [{currencyCode}] has insufficient balance | 32126: Section ID is required for placing this order | 38121: Duplicate permanent item exists | 32124: Invalid currency namespace | 32127: Discount code [{code}] can't be used on this item: {tips} | 32128: Discount code [{code}] can not be used with other code together | 32129: Can't use discount code on free order | 32130: The total discount amount cannot exceed the order price)

        403: Forbidden - ErrorEntity (20016: action is banned)

        404: Not Found - ErrorEntity (30341: Item [{itemId}] does not exist in namespace [{namespace}] | 30142: Published store does not exist in namespace [{namespace}] | 36141: Currency [{currencyCode}] does not exist in namespace [{namespace}] | 49147: Published season does not exist | 1100001: record not found: inventory | 37142: Code [{code}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (20024: insufficient inventory capacity (max. slots) | 32175: Exceed item [{itemId}] max count [{maxCount}] per user | 32176: Exceed item [{itemId}] max count [{maxCount}] | 32178: User [{userId}] already owned all durable items in flexible bundle [{bundleId}], namespace: [{namespace}] | 31177: Permanent item already owned | 49183: Pass item does not match published season pass | 49184: Tier item does not match published season tier | 49185: Season has not started | 49186: Pass already owned | 49187: Exceed max tier count | 20006: optimistic lock | 37172: Campaign [{campaignId}] is inactive in namespace [{namespace}] | 37173: Code [{code}] is inactive in namespace [{namespace}] | 37174: Exceeded max redeem count per code [{maxCount}] | 37175: Exceeded max redeem count per code per user [{maxCount}] | 37177: Code redemption not started | 37178: Code redemption already ended | 37179: Exceeded max redeem count per campaign per user [{maxCount}] | 30771: Item [{itemId}] not found in User Section [{sectionId}], UserId [{userId}], Namespace [{namespace}] | 30772: Section [{sectionId}] is not available or expired)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 201:
            return OrderInfo.create_from_dict(content), None
        if code == 400:
            return None, ErrorEntity.create_from_dict(content)
        if code == 403:
            return None, ErrorEntity.create_from_dict(content)
        if code == 404:
            return None, ErrorEntity.create_from_dict(content)
        if code == 409:
            return None, ErrorEntity.create_from_dict(content)
        if code == 422:
            return None, ValidationErrorEntity.create_from_dict(content)

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls, body: AdminOrderCreate, namespace: str, user_id: str, **kwargs
    ) -> AdminCreateUserOrder:
        instance = cls()
        instance.body = body
        instance.namespace = namespace
        instance.user_id = user_id
        if x_flight_id := kwargs.get("x_flight_id", None):
            instance.x_flight_id = x_flight_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> AdminCreateUserOrder:
        instance = cls()
        if "body" in dict_ and dict_["body"] is not None:
            instance.body = AdminOrderCreate.create_from_dict(
                dict_["body"], include_empty=include_empty
            )
        elif include_empty:
            instance.body = AdminOrderCreate()
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "body": "body",
            "namespace": "namespace",
            "userId": "user_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "body": True,
            "namespace": True,
            "userId": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: operation.j2

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Gaming Services Platform Service

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import ApiError, ApiResponse
from accelbyte_py_sdk.core import Operation
from accelbyte_py_sdk.core import HeaderStr
from accelbyte_py_sdk.core import HttpResponse
from accelbyte_py_sdk.core import deprecated

from ...models import ErrorEntity
from ...models import ItemTypeConfigCreate
from ...models import ValidationErrorEntity


class CreateItemTypeConfig(Operation):
    """Create an Item Type Config (createItemTypeConfig)

    [Not supported yet in AGS Shared Cloud] This API is used to create an item type config.

    *  Returns : item type config data

    Properties:
        url: /platform/admin/items/configs

        method: POST

        tags: ["Item"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ItemTypeConfigCreate in body

    Responses:
        201: Created - (item type config created)

        400: Bad Request - ErrorEntity (30336: Item type [{itemType}] does not support)

        409: Conflict - ErrorEntity (30371: Item type config for type [{itemType}] and clazz [{clazz}] already exists)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """

    # region fields

    _url: str = "/platform/admin/items/configs"
    _path: str = "/platform/admin/items/configs"
    _base_path: str = ""
    _method: str = "POST"
    _consumes: List[str] = ["application/json"]
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"]]
    _location_query: str = None

    service_name: Optional[str] = "platform"

    body: ItemTypeConfigCreate  # REQUIRED in [body]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def path(self) -> str:
        return self._path

    @property
    def base_path(self) -> str:
        return self._base_path

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "body": self.get_body_params(),
        }

    def get_body_params(self) -> Any:
        if not hasattr(self, "body") or self.body is None:
            return None
        return self.body.to_dict()

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_body(self, value: ItemTypeConfigCreate) -> CreateItemTypeConfig:
        self.body = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "body") and self.body:
            result["body"] = self.body.to_dict(include_empty=include_empty)
        elif include_empty:
            result["body"] = ItemTypeConfigCreate()
        return result

    # endregion to methods

    # region response methods

    class Response(ApiResponse):
        data_201: Optional[str] = None
        error_400: Optional[ErrorEntity] = None
        error_409: Optional[ErrorEntity] = None
        error_422: Optional[ValidationErrorEntity] = None

        def ok(self) -> CreateItemTypeConfig.Response:
            if self.error_400 is not None:
                err = self.error_400.translate_to_api_error()
                exc = err.to_exception()
                if exc is not None:
                    raise exc  # pylint: disable=raising-bad-type
            if self.error_409 is not None:
                err = self.error_409.translate_to_api_error()
                exc = err.to_exception()
                if exc is not None:
                    raise exc  # pylint: disable=raising-bad-type
            if self.error_422 is not None:
                err = self.error_422.translate_to_api_error()
                exc = err.to_exception()
                if exc is not None:
                    raise exc  # pylint: disable=raising-bad-type
            return self

        def __iter__(self):
            if self.data_201 is not None:
                yield self.data_201
                yield None
            elif self.error_400 is not None:
                yield None
                yield self.error_400
            elif self.error_409 is not None:
                yield None
                yield self.error_409
            elif self.error_422 is not None:
                yield None
                yield self.error_422
            else:
                yield None
                yield self.error

    # noinspection PyMethodMayBeStatic
    def parse_response(self, code: int, content_type: str, content: Any) -> Response:
        """Parse the given response.

        201: Created - (item type config created)

        400: Bad Request - ErrorEntity (30336: Item type [{itemType}] does not support)

        409: Conflict - ErrorEntity (30371: Item type config for type [{itemType}] and clazz [{clazz}] already exists)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        result = CreateItemTypeConfig.Response()

        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )

        if error is not None:
            if not error.is_no_content():
                result.error = ApiError.create_from_http_response(error)
        else:
            code, content_type, content = pre_processed_response

            if code == 201:
                result.data_201 = content
            elif code == 400:
                result.error_400 = ErrorEntity.create_from_dict(content)
                result.error = result.error_400.translate_to_api_error()
            elif code == 409:
                result.error_409 = ErrorEntity.create_from_dict(content)
                result.error = result.error_409.translate_to_api_error()
            elif code == 422:
                result.error_422 = ValidationErrorEntity.create_from_dict(content)
                result.error = result.error_422.translate_to_api_error()
            else:
                result.error = ApiError.create_from_http_response(
                    HttpResponse.create_undocumented_response(
                        code=code, content_type=content_type, content=content
                    )
                )

        result.status_code = str(code)
        result.content_type = content_type

        if 400 <= code <= 599 or result.error is not None:
            result.is_success = False

        return result

    # noinspection PyMethodMayBeStatic
    @deprecated
    def parse_response_x(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[
        Union[None, Optional[str]],
        Union[None, ErrorEntity, HttpResponse, ValidationErrorEntity],
    ]:
        """Parse the given response.

        201: Created - (item type config created)

        400: Bad Request - ErrorEntity (30336: Item type [{itemType}] does not support)

        409: Conflict - ErrorEntity (30371: Item type config for type [{itemType}] and clazz [{clazz}] already exists)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 201:
            return content, None
        if code == 400:
            return None, ErrorEntity.create_from_dict(content)
        if code == 409:
            return None, ErrorEntity.create_from_dict(content)
        if code == 422:
            return None, ValidationErrorEntity.create_from_dict(content)

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(cls, body: ItemTypeConfigCreate, **kwargs) -> CreateItemTypeConfig:
        instance = cls()
        instance.body = body
        if x_flight_id := kwargs.get("x_flight_id", None):
            instance.x_flight_id = x_flight_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> CreateItemTypeConfig:
        instance = cls()
        if "body" in dict_ and dict_["body"] is not None:
            instance.body = ItemTypeConfigCreate.create_from_dict(
                dict_["body"], include_empty=include_empty
            )
        elif include_empty:
            instance.body = ItemTypeConfigCreate()
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "body": "body",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "body": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model
from accelbyte_py_sdk.core import StrEnum


class FeaturesToCheckEnum(StrEnum):
    CAMPAIGN = "CAMPAIGN"
    CATALOG = "CATALOG"
    DLC = "DLC"
    ENTITLEMENT = "ENTITLEMENT"
    IAP = "IAP"
    REWARD = "REWARD"


class ChangeStatusItemRequest(Model):
    """Change status item request (ChangeStatusItemRequest)

    Properties:
        features_to_check: (featuresToCheck) OPTIONAL Union[List[str], List[FeaturesToCheckEnum]]
    """

    # region fields

    features_to_check: Union[List[str], List[FeaturesToCheckEnum]]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_features_to_check(
        self, value: Union[List[str], List[FeaturesToCheckEnum]]
    ) -> ChangeStatusItemRequest:
        self.features_to_check = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "features_to_check"):
            result["featuresToCheck"] = [str(i0) for i0 in self.features_to_check]
        elif include_empty:
            result["featuresToCheck"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        features_to_check: Optional[Union[List[str], List[FeaturesToCheckEnum]]] = None,
        **kwargs,
    ) -> ChangeStatusItemRequest:
        instance = cls()
        if features_to_check is not None:
            instance.features_to_check = features_to_check
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ChangeStatusItemRequest:
        instance = cls()
        if not dict_:
            return instance
        if "featuresToCheck" in dict_ and dict_["featuresToCheck"] is not None:
            instance.features_to_check = [str(i0) for i0 in dict_["featuresToCheck"]]
        elif include_empty:
            instance.features_to_check = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ChangeStatusItemRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ChangeStatusItemRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ChangeStatusItemRequest,
        List[ChangeStatusItemRequest],
        Dict[Any, ChangeStatusItemRequest],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "featuresToCheck": "features_to_check",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "featuresToCheck": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "featuresToCheck": [
                "CAMPAIGN",
                "CATALOG",
                "DLC",
                "ENTITLEMENT",
                "IAP",
                "REWARD",
            ],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model


class DiscountCategory(Model):
    """Discount category (DiscountCategory)

    Properties:
        category_path: (categoryPath) REQUIRED str

        include_sub_categories: (includeSubCategories) OPTIONAL bool
    """

    # region fields

    category_path: str  # REQUIRED
    include_sub_categories: bool  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_category_path(self, value: str) -> DiscountCategory:
        self.category_path = value
        return self

    def with_include_sub_categories(self, value: bool) -> DiscountCategory:
        self.include_sub_categories = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "category_path"):
            result["categoryPath"] = str(self.category_path)
        elif include_empty:
            result["categoryPath"] = ""
        if hasattr(self, "include_sub_categories"):
            result["includeSubCategories"] = bool(self.include_sub_categories)
        elif include_empty:
            result["includeSubCategories"] = False
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls, category_path: str, include_sub_categories: Optional[bool] = None, **kwargs
    ) -> DiscountCategory:
        instance = cls()
        instance.category_path = category_path
        if include_sub_categories is not None:
            instance.include_sub_categories = include_sub_categories
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> DiscountCategory:
        instance = cls()
        if not dict_:
            return instance
        if "categoryPath" in dict_ and dict_["categoryPath"] is not None:
            instance.category_path = str(dict_["categoryPath"])
        elif include_empty:
            instance.category_path = ""
        if (
            "includeSubCategories" in dict_
            and dict_["includeSubCategories"] is not None
        ):
            instance.include_sub_categories = bool(dict_["includeSubCategories"])
        elif include_empty:
            instance.include_sub_categories = False
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, DiscountCategory]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[DiscountCategory]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[DiscountCategory, List[DiscountCategory], Dict[Any, DiscountCategory]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "categoryPath": "category_path",
            "includeSubCategories": "include_sub_categories",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "categoryPath": True,
            "includeSubCategories": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model
from accelbyte_py_sdk.core import StrEnum

from ..models.entitlement_origin_sync_result import EntitlementOriginSyncResult
from ..models.entitlement_summary import EntitlementSummary
from ..models.platform_reward import PlatformReward
from ..models.revocation_result import RevocationResult
from ..models.revoke_result import RevokeResult


class EntitlementOriginSyncStatusEnum(StrEnum):
    NOT_SYNCED = "NOT_SYNCED"
    SYNCED = "SYNCED"
    SYNCED_FAILED = "SYNCED_FAILED"


class PlatformEnum(StrEnum):
    EPICGAMES = "EPICGAMES"
    OCULUS = "OCULUS"
    PSN = "PSN"
    STEAM = "STEAM"
    XBOX = "XBOX"


class StatusEnum(StrEnum):
    FULFILLED = "FULFILLED"
    REVOKED = "REVOKED"
    REVOKE_FAILED = "REVOKE_FAILED"


class UserDLCRecord(Model):
    """User DLC record (UserDLCRecord)

    Properties:
        dlc_reward_version: (dlcRewardVersion) OPTIONAL int

        entitlement_origin_sync_result: (entitlementOriginSyncResult) OPTIONAL List[EntitlementOriginSyncResult]

        entitlement_origin_sync_status: (entitlementOriginSyncStatus) OPTIONAL Union[str, EntitlementOriginSyncStatusEnum]

        entitlement_summaries: (entitlementSummaries) OPTIONAL List[EntitlementSummary]

        id_: (id) OPTIONAL str

        metadata: (metadata) OPTIONAL Dict[str, Any]

        namespace: (namespace) OPTIONAL str

        obtained_at: (obtainedAt) OPTIONAL str

        platform: (platform) OPTIONAL Union[str, PlatformEnum]

        revocation_result: (revocationResult) OPTIONAL RevocationResult

        revoke_results: (revokeResults) OPTIONAL List[RevokeResult]

        revoked_at: (revokedAt) OPTIONAL str

        rewards: (rewards) OPTIONAL List[PlatformReward]

        sources: (sources) OPTIONAL List[str]

        status: (status) OPTIONAL Union[str, StatusEnum]

        transaction_id: (transactionId) OPTIONAL str

        user_id: (userId) OPTIONAL str

        version: (version) OPTIONAL int
    """

    # region fields

    dlc_reward_version: int  # OPTIONAL
    entitlement_origin_sync_result: List[EntitlementOriginSyncResult]  # OPTIONAL
    entitlement_origin_sync_status: Union[
        str, EntitlementOriginSyncStatusEnum
    ]  # OPTIONAL
    entitlement_summaries: List[EntitlementSummary]  # OPTIONAL
    id_: str  # OPTIONAL
    metadata: Dict[str, Any]  # OPTIONAL
    namespace: str  # OPTIONAL
    obtained_at: str  # OPTIONAL
    platform: Union[str, PlatformEnum]  # OPTIONAL
    revocation_result: RevocationResult  # OPTIONAL
    revoke_results: List[RevokeResult]  # OPTIONAL
    revoked_at: str  # OPTIONAL
    rewards: List[PlatformReward]  # OPTIONAL
    sources: List[str]  # OPTIONAL
    status: Union[str, StatusEnum]  # OPTIONAL
    transaction_id: str  # OPTIONAL
    user_id: str  # OPTIONAL
    version: int  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_dlc_reward_version(self, value: int) -> UserDLCRecord:
        self.dlc_reward_version = value
        return self

    def with_entitlement_origin_sync_result(
        self, value: List[EntitlementOriginSyncResult]
    ) -> UserDLCRecord:
        self.entitlement_origin_sync_result = value
        return self

    def with_entitlement_origin_sync_status(
        self, value: Union[str, EntitlementOriginSyncStatusEnum]
    ) -> UserDLCRecord:
        self.entitlement_origin_sync_status = value
        return self

    def with_entitlement_summaries(
        self, value: List[EntitlementSummary]
    ) -> UserDLCRecord:
        self.entitlement_summaries = value
        return self

    def with_id(self, value: str) -> UserDLCRecord:
        self.id_ = value
        return self

    def with_metadata(self, value: Dict[str, Any]) -> UserDLCRecord:
        self.metadata = value
        return self

    def with_namespace(self, value: str) -> UserDLCRecord:
        self.namespace = value
        return self

    def with_obtained_at(self, value: str) -> UserDLCRecord:
        self.obtained_at = value
        return self

    def with_platform(self, value: Union[str, PlatformEnum]) -> UserDLCRecord:
        self.platform = value
        return self

    def with_revocation_result(self, value: RevocationResult) -> UserDLCRecord:
        self.revocation_result = value
        return self

    def with_revoke_results(self, value: List[RevokeResult]) -> UserDLCRecord:
        self.revoke_results = value
        return self

    def with_revoked_at(self, value: str) -> UserDLCRecord:
        self.revoked_at = value
        return self

    def with_rewards(self, value: List[PlatformReward]) -> UserDLCRecord:
        self.rewards = value
        return self

    def with_sources(self, value: List[str]) -> UserDLCRecord:
        self.sources = value
        return self

    def with_status(self, value: Union[str, StatusEnum]) -> UserDLCRecord:
        self.status = value
        return self

    def with_transaction_id(self, value: str) -> UserDLCRecord:
        self.transaction_id = value
        return self

    def with_user_id(self, value: str) -> UserDLCRecord:
        self.user_id = value
        return self

    def with_version(self, value: int) -> UserDLCRecord:
        self.version = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "dlc_reward_version"):
            result["dlcRewardVersion"] = int(self.dlc_reward_version)
        elif include_empty:
            result["dlcRewardVersion"] = 0
        if hasattr(self, "entitlement_origin_sync_result"):
            result["entitlementOriginSyncResult"] = [
                i0.to_dict(include_empty=include_empty)
                for i0 in self.entitlement_origin_sync_result
            ]
        elif include_empty:
            result["entitlementOriginSyncResult"] = []
        if hasattr(self, "entitlement_origin_sync_status"):
            result["entitlementOriginSyncStatus"] = str(
                self.entitlement_origin_sync_status
            )
        elif include_empty:
            result["entitlementOriginSyncStatus"] = Union[
                str, EntitlementOriginSyncStatusEnum
            ]()
        if hasattr(self, "entitlement_summaries"):
            result["entitlementSummaries"] = [
                i0.to_dict(include_empty=include_empty)
                for i0 in self.entitlement_summaries
            ]
        elif include_empty:
            result["entitlementSummaries"] = []
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "metadata"):
            result["metadata"] = {str(k0): v0 for k0, v0 in self.metadata.items()}
        elif include_empty:
            result["metadata"] = {}
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "obtained_at"):
            result["obtainedAt"] = str(self.obtained_at)
        elif include_empty:
            result["obtainedAt"] = ""
        if hasattr(self, "platform"):
            result["platform"] = str(self.platform)
        elif include_empty:
            result["platform"] = Union[str, PlatformEnum]()
        if hasattr(self, "revocation_result"):
            result["revocationResult"] = self.revocation_result.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["revocationResult"] = RevocationResult()
        if hasattr(self, "revoke_results"):
            result["revokeResults"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.revoke_results
            ]
        elif include_empty:
            result["revokeResults"] = []
        if hasattr(self, "revoked_at"):
            result["revokedAt"] = str(self.revoked_at)
        elif include_empty:
            result["revokedAt"] = ""
        if hasattr(self, "rewards"):
            result["rewards"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.rewards
            ]
        elif include_empty:
            result["rewards"] = []
        if hasattr(self, "sources"):
            result["sources"] = [str(i0) for i0 in self.sources]
        elif include_empty:
            result["sources"] = []
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "transaction_id"):
            result["transactionId"] = str(self.transaction_id)
        elif include_empty:
            result["transactionId"] = ""
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "version"):
            result["version"] = int(self.version)
        elif include_empty:
            result["version"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        dlc_reward_version: Optional[int] = None,
        entitlement_origin_sync_result: Optional[
            List[EntitlementOriginSyncResult]
        ] = None,
        entitlement_origin_sync_status: Optional[
            Union[str, EntitlementOriginSyncStatusEnum]
        ] = None,
        entitlement_summaries: Optional[List[EntitlementSummary]] = None,
        id_: Optional[str] = None,
        metadata: Optional[Dict[str, Any]] = None,
        namespace: Optional[str] = None,
        obtained_at: Optional[str] = None,
        platform: Optional[Union[str, PlatformEnum]] = None,
        revocation_result: Optional[RevocationResult] = None,
        revoke_results: Optional[List[RevokeResult]] = None,
        revoked_at: Optional[str] = None,
        rewards: Optional[List[PlatformReward]] = None,
        sources: Optional[List[str]] = None,
        status: Optional[Union[str, StatusEnum]] = None,
        transaction_id: Optional[str] = None,
        user_id: Optional[str] = None,
        version: Optional[int] = None,
        **kwargs,
    ) -> UserDLCRecord:
        instance = cls()
        if dlc_reward_version is not None:
            instance.dlc_reward_version = dlc_reward_version
        if entitlement_origin_sync_result is not None:
            instance.entitlement_origin_sync_result = entitlement_origin_sync_result
        if entitlement_origin_sync_status is not None:
            instance.entitlement_origin_sync_status = entitlement_origin_sync_status
        if entitlement_summaries is not None:
            instance.entitlement_summaries = entitlement_summaries
        if id_ is not None:
            instance.id_ = id_
        if metadata is not None:
            instance.metadata = metadata
        if namespace is not None:
            instance.namespace = namespace
        if obtained_at is not None:
            instance.obtained_at = obtained_at
        if platform is not None:
            instance.platform = platform
        if revocation_result is not None:
            instance.revocation_result = revocation_result
        if revoke_results is not None:
            instance.revoke_results = revoke_results
        if revoked_at is not None:
            instance.revoked_at = revoked_at
        if rewards is not None:
            instance.rewards = rewards
        if sources is not None:
            instance.sources = sources
        if status is not None:
            instance.status = status
        if transaction_id is not None:
            instance.transaction_id = transaction_id
        if user_id is not None:
            instance.user_id = user_id
        if version is not None:
            instance.version = version
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> UserDLCRecord:
        instance = cls()
        if not dict_:
            return instance
        if "dlcRewardVersion" in dict_ and dict_["dlcRewardVersion"] is not None:
            instance.dlc_reward_version = int(dict_["dlcRewardVersion"])
        elif include_empty:
            instance.dlc_reward_version = 0
        if (
            "entitlementOriginSyncResult" in dict_
            and dict_["entitlementOriginSyncResult"] is not None
        ):
            instance.entitlement_origin_sync_result = [
                EntitlementOriginSyncResult.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["entitlementOriginSyncResult"]
            ]
        elif include_empty:
            instance.entitlement_origin_sync_result = []
        if (
            "entitlementOriginSyncStatus" in dict_
            and dict_["entitlementOriginSyncStatus"] is not None
        ):
            instance.entitlement_origin_sync_status = str(
                dict_["entitlementOriginSyncStatus"]
            )
        elif include_empty:
            instance.entitlement_origin_sync_status = Union[
                str, EntitlementOriginSyncStatusEnum
            ]()
        if (
            "entitlementSummaries" in dict_
            and dict_["entitlementSummaries"] is not None
        ):
            instance.entitlement_summaries = [
                EntitlementSummary.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["entitlementSummaries"]
            ]
        elif include_empty:
            instance.entitlement_summaries = []
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "metadata" in dict_ and dict_["metadata"] is not None:
            instance.metadata = {str(k0): v0 for k0, v0 in dict_["metadata"].items()}
        elif include_empty:
            instance.metadata = {}
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "obtainedAt" in dict_ and dict_["obtainedAt"] is not None:
            instance.obtained_at = str(dict_["obtainedAt"])
        elif include_empty:
            instance.obtained_at = ""
        if "platform" in dict_ and dict_["platform"] is not None:
            instance.platform = str(dict_["platform"])
        elif include_empty:
            instance.platform = Union[str, PlatformEnum]()
        if "revocationResult" in dict_ and dict_["revocationResult"] is not None:
            instance.revocation_result = RevocationResult.create_from_dict(
                dict_["revocationResult"], include_empty=include_empty
            )
        elif include_empty:
            instance.revocation_result = RevocationResult()
        if "revokeResults" in dict_ and dict_["revokeResults"] is not None:
            instance.revoke_results = [
                RevokeResult.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["revokeResults"]
            ]
        elif include_empty:
            instance.revoke_results = []
        if "revokedAt" in dict_ and dict_["revokedAt"] is not None:
            instance.revoked_at = str(dict_["revokedAt"])
        elif include_empty:
            instance.revoked_at = ""
        if "rewards" in dict_ and dict_["rewards"] is not None:
            instance.rewards = [
                PlatformReward.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["rewards"]
            ]
        elif include_empty:
            instance.rewards = []
        if "sources" in dict_ and dict_["sources"] is not None:
            instance.sources = [str(i0) for i0 in dict_["sources"]]
        elif include_empty:
            instance.sources = []
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if "transactionId" in dict_ and dict_["transactionId"] is not None:
            instance.transaction_id = str(dict_["transactionId"])
        elif include_empty:
            instance.transaction_id = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "version" in dict_ and dict_["version"] is not None:
            instance.version = int(dict_["version"])
        elif include_empty:
            instance.version = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, UserDLCRecord]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[UserDLCRecord]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[UserDLCRecord, List[UserDLCRecord], Dict[Any, UserDLCRecord]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "dlcRewardVersion": "dlc_reward_version",
            "entitlementOriginSyncResult": "entitlement_origin_sync_result",
            "entitlementOriginSyncStatus": "entitlement_origin_sync_status",
            "entitlementSummaries": "entitlement_summaries",
            "id": "id_",
            "metadata": "metadata",
            "namespace": "namespace",
            "obtainedAt": "obtained_at",
            "platform": "platform",
            "revocationResult": "revocation_result",
            "revokeResults": "revoke_results",
            "revokedAt": "revoked_at",
            "rewards": "rewards",
            "sources": "sources",
            "status": "status",
            "transactionId": "transaction_id",
            "userId": "user_id",
            "version": "version",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "dlcRewardVersion": False,
            "entitlementOriginSyncResult": False,
            "entitlementOriginSyncStatus": False,
            "entitlementSummaries": False,
            "id": False,
            "metadata": False,
            "namespace": False,
            "obtainedAt": False,
            "platform": False,
            "revocationResult": False,
            "revokeResults": False,
            "revokedAt": False,
            "rewards": False,
            "sources": False,
            "status": False,
            "transactionId": False,
            "userId": False,
            "version": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "entitlementOriginSyncStatus": ["NOT_SYNCED", "SYNCED", "SYNCED_FAILED"],
            "platform": ["EPICGAMES", "OCULUS", "PSN", "STEAM", "XBOX"],
            "status": ["FULFILLED", "REVOKED", "REVOKE_FAILED"],
        }

    # endregion static methods
